package com.jh.boot.email;

import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Objects;

import javax.mail.internet.MimeMessage;
import javax.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.util.StringUtils;

/**
 * The Class LocalEmailService.
 */
public class LocalEmailService implements EmailService {

	/** The Constant LOG. */
	private static final Logger LOG = LoggerFactory.getLogger(LocalEmailService.class);
	
	/** The repo. */
	@Autowired
	private EmailRepository repo;
	
	/** The mail sender. */
	@Autowired 
	private JavaMailSender mailSender;
	
	/** The email sender address. */
	@Value("${email.sender.from:}")
	private String emailSenderAddress;
	
	
	/**
	 * @param email
	 * @return
	 * @see com.jh.module.email.EmailService#sendEmail(com.jh.module.email.Email)
	 */
	@Override
	@Transactional
	public Email sendEmail(Email email) {
		email.setToBeSent(true);
		email.setLocked(false);
		
		if (!StringUtils.hasText(email.getEmailFrom())) {
			email.setEmailFrom(emailSenderAddress);
		}
		
		if (email.getRecipients() != null) {
			for (EmailRecipient er : email.getRecipients()) {
				er.setEmail(email);
			}
		}
		if (email.getAttachments() != null) {
			for (EmailAttachment ea : email.getAttachments()) {
				ea.setEmail(email);
			}
		}
		
		repo.save(email);
		
		//TODO
		doSendEmail(email);
		
		return email;
	}

		
//	@Scheduled(fixedDelayString = "${localEmail.scheduled.interval.ms:120000}")
//	@Transactional
//	public void scheduled() {
//		List<Email> emails = repo.fetchEmailsToSend();
//		if (emails == null || emails.isEmpty()) {
//			return;
//		}
//		LOG.info("Going to send {} emails", emails.size());
//		for (Email email : emails) {
//			doSendEmail(email);
//		}
//	}


	/**
	 * TODO.
	 *
	 * @param email the email
	 */
	private void doSendEmail(Email email) {
		MimeMessage msg = mailSender.createMimeMessage();
		
		try {
			MimeMessageHelper helper = new MimeMessageHelper(msg, true);
			
			//send receipt?
			if (Objects.equals(email.getReceipt(), true)) {
                msg.addHeader("Disposition-Notification-To", email.getEmailFrom());
			}
			
			//TODO add tracking-number
			msg.addHeader("X-Email-RefId", String.valueOf(email.getId()));

			//use properly configured (reverse DNS) sender address?
			if (StringUtils.hasText(emailSenderAddress)) {
				helper.setFrom(emailSenderAddress);
				if (StringUtils.hasText(email.getFrom())) {
					helper.setReplyTo(email.getFrom());
				}
			} else {
				helper.setFrom(email.getFrom());
			}
			
			List<String> to = new ArrayList<>();
			List<String> cc = new ArrayList<>();
			List<String> bcc = new ArrayList<>();
			if (email.getRecipients() != null) {
				for (EmailRecipient r : email.getRecipients()) {
					RecipientType rt = r.getRecipientType();
					
					switch(rt) { 
					case BCC:
						bcc.add(r.getAddress());
						break;
					case CC:
						cc.add(r.getAddress());
						break;
					case TO:
					default:
						to.add(r.getAddress());
					}
				}
			}
			String[] s = new String[0];
			helper.setTo(to.toArray(s));
			helper.setCc(cc.toArray(s));
			helper.setBcc(bcc.toArray(s));

			helper.setSubject(email.getSubject());
			helper.setText(email.getText());
			if (StringUtils.hasText(email.getHtml())) {
				helper.setText(email.getHtml(), true);
			}
			
			if (email.getAttachments() != null) {	
				for (EmailAttachment a : email.getAttachments()) {
					if (a.getData() != null) {
						byte[] bytes = Base64.getDecoder().decode(a.getData().getBytes());
						if (StringUtils.hasText(a.getCid())) {
                            helper.addInline(a.getCid(),  new ByteArrayResource(bytes), a.getMimeType());
						} else {
						    helper.addAttachment(a.getName(), new ByteArrayResource(bytes), a.getMimeType());
						}
					}
				}
			}
			
			mailSender.send(msg);
			email.setStatus("SENT");
			
		} catch (Exception e) {
			e.printStackTrace();
			email.setStatus("FAILED");
			email.setError(e.getMessage());
		}
		email.setToBeSent(false);
		email.setLocked(false);
	}
	
}
