package com.jh.boot.security.repository;

import javax.persistence.Query;

import org.springframework.util.StringUtils;

import com.jh.boot.jpa.AbstractHibernateRepository;
import com.jh.boot.security.PasswordHash;
import com.jh.boot.security.PasswordUtils;
import com.jh.boot.security.model.AppUser;
import com.jh.boot.security.model.ResetPasswordToken;


public class AppUserRepository extends AbstractHibernateRepository {

	public AppUser findByLogin(String username) {
		if (!StringUtils.hasText(username)) {
			return null;
		}
		Query q = entityManager.createQuery("select au from AppUser au where lower(au.email) = :email");
		q.setParameter("email", username.trim().toLowerCase());
		return singleResult(q);
	}
	
	
	public AppUser fetchByLoginDetached(String username) {
		if (!StringUtils.hasText(username)) {
			return null;
		}
		Query q = entityManager.createQuery("select au from AppUser au left join fetch au.roles where lower(au.email) = :email");
		q.setParameter("email", username.trim().toLowerCase());
		AppUser user = singleResult(q);
		if (user == null) {
			return null;
		}
		entityManager.detach(user);
		return user;
	}
	
	
	public AppUser registerUser(String username, String password) {
		AppUser user = new AppUser();
		user.setEmail(username);
		
		PasswordHash hash = PasswordUtils.hashPassword(password);
		user.setPassword(hash.getHash());
		user.setPasswordSalt(hash.getSalt());
		
		entityManager.persist(user);
		
		return user;
	}


	public ResetPasswordToken saveResetPasswordToken(ResetPasswordToken token) {
		entityManager.persist(token);
		return token;
	}
	
	public ResetPasswordToken findResetPasswordToken(String login, String token) {
		if (!StringUtils.hasText(login) || StringUtils.hasText(token)) {
			return null;
		}
		Query q = entityManager.createQuery("select au from ResetPasswordToken au where lower(au.login) = :login and au.token = :token and (au.used is null or au.used = :used)");
		q.setParameter("login", login.trim().toLowerCase());
		q.setParameter("token", token.trim().toLowerCase());
		q.setParameter("used", Boolean.TRUE);
		return singleResult(q);
	}
	
}
